from .BaseHandler import BaseHandler
from ..models.Repository import Repository


class CreateGitignoreHandler(BaseHandler):
    def process(self, request: Repository):
        # Prompt for .gitignore content
        ignored_content = input("Please enter files/folders you want to add to .gitignore, separated by comma:\n")
        clean_content_list = [content.strip() for content in ignored_content.split(",")]

        # Create .gitignore file & Write the content to the file
        with open(f"{request.working_directory}/.gitignore", "a") as gitignore:
            for line in clean_content_list:
                gitignore.write(f"\n{line}\n")

        self.move_to_next_handler(request)



import subprocess


def run_command(command):
    # shell=True: This argument specifies that the command should be executed through the shell.
    # This allows you to run complex shell commands and use shell features like wildcard expansion.

    # check=True: If set to True, this argument makes the subprocess.run function raise a subprocess.CalledProcessError
    # exception if the command returns a non-zero exit code, indicating an error.

    # stdout=subprocess.PIPE: This argument redirects the standard output (stdout) of the command to a pipe,
    # allowing the output to be captured and processed in the Python script.

    # stderr=subprocess.PIPE: Similar to stdout. But it redirects standard error to a pipe instead.
    result = subprocess.run(command, shell=True, check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    return result.stdout.decode().strip()  # Decode the byte string returned by result.stdout to a regular string.



class Repository:
    def __init__(self,
                 repo_name=None,
                 github_username=None,
                 github_token=None,
                 working_directory=None,
                 include_gitignore=None,
                 is_private=None):
        self.repo_name = repo_name
        self.github_username = github_username
        self.github_token = github_token
        self.working_directory = working_directory
        self.include_gitignore = True if include_gitignore.lower() == "y" else False
        self.is_private = True if is_private.lower() == "y" else False

    def __str__(self):
        return f"""
        repo_name: {self.repo_name},
        github_username: {self.github_username}
        github_token: {self.github_token}
        working_directory: {self.working_directory}"""



import enum


class Command(enum.Enum):
    CD = "cd"
    GIT_INIT = "git init"
    GIT_COMMIT = "git commit -m"
    GIT_STAGE_ALL = "git add ."
    GIT_INITIAL_PUSH = "git push --set-upstream origin main"
    ADD_REMOTE_URL_TO_LOCAL_REPO = "git remote add origin"



from .models.Repository import Repository
from .handlers.CreateRemoteRepoHandler import CreateRemoteRepoHandler
from .handlers.CreateGitignoreHandler import CreateGitignoreHandler
import argparse
import os

username = os.getenv('GITHUB_USERNAME')
token = os.getenv('GITHUB_TOKEN')


def validate_request(request):
    if request.repo_name is None:
        raise ValueError("Remote repository name is required")
    elif request.working_directory is None or not os.path.exists(request.working_directory):
        raise ValueError("Path to working directory is either not provided or invalid")
    elif request.github_token is None or request.github_username is None:
        raise ValueError("No GitHub token or username found. Please check your global variables or manually enter them")


def setup_request_commandline() -> Repository:
    parser = argparse.ArgumentParser()
    parser.add_argument("-rn", "--repo-name",
                        help="The name of the remote repository you want to create")
    parser.add_argument("-gu", "--github-username",
                        help="The username of your Github account",
                        default=username)
    parser.add_argument("-t", "--github-token",
                        help="Your Github authorization token",
                        default=token)
    parser.add_argument("-d", "--working-directory",
                        help="The absolute path to local working directory, where the repo should be initialized")
    parser.add_argument("-i", "--include-gitignore",
                        help="Enter y if you want to add .gitignore to your repo, otherwise enter n",
                        default="n")
    parser.add_argument("-p", "--private",
                        help="Enter y if you want to make the remote repo private, otherwise enter n",
                        default="n")

    try:
        args = parser.parse_args()
        repo = Repository(repo_name=args.repo_name,
                          github_username=args.github_username,
                          github_token=args.github_token,
                          working_directory=args.working_directory,
                          include_gitignore=args.include_gitignore,
                          is_private=args.private)
        validate_request(repo)
        return repo

    except Exception as e:
        print(f'Error: {e}')
        quit()


def main():
    repo = setup_request_commandline()

    create_repo_handler = CreateRemoteRepoHandler()

    if repo.include_gitignore:
        create_gitignore_handler = CreateGitignoreHandler()
        create_gitignore_handler.set_next_handler(create_repo_handler)
        try:
            create_gitignore_handler.process(repo)
        except Exception as e:
            print(e)
    else:
        try:
            create_repo_handler.process(repo)
        except Exception as e:
            print(e)


if __name__ == '__main__':
    main()



from .BaseHandler import BaseHandler
from ..enum.Command import Command
from ..helpers.run_command import run_command as rc
from ..models.Repository import Repository
from ..enum.GitHub_API import GitHubAPI as apis
import requests


class CreateRemoteRepoHandler(BaseHandler):
    def process(self, request: Repository):
        # Create local repo, stage the changes, and commit.
        rc(f"{Command.CD.value} {request.working_directory} "
           f"&& {Command.GIT_INIT.value} "
           f"&& {Command.GIT_STAGE_ALL.value} "
           f"&& {Command.GIT_COMMIT.value} \'Initial commit\'")

        # Create a new repo on GitHub
        res = requests.post(
            f"{apis.BASE.value}{apis.CREATE_REPO_AUTH_USER.value}",
            json={'name': request.repo_name, 'private': request.is_private},
            auth=(request.github_username, request.github_token)
        )
        if res.status_code != 201:
            print(f'remote repo creation failed. {res.json()}')
            raise Exception("Github Api call fails")
        remote_clone_url = res.json()['clone_url']

        # Add the remote clone url to the local git repo and push the local changes to the remote origin
        rc(f"{Command.CD.value} {request.working_directory} "
           f"&& {Command.ADD_REMOTE_URL_TO_LOCAL_REPO.value} {remote_clone_url} "
           f"&& {Command.GIT_INITIAL_PUSH.value}")

        self.move_to_next_handler(request)



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read()

setup(
    name='create-remote-repo',
    version="1.0.0",
    author='Kim Chung',
    author_email='kchung0802@gmail.com',
    license='MIT License',
    description='This tool automate the process of creating a repo from your current working directory and pushing '
                'it to your remote Github.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='',
    packages=find_packages(),
    install_requires=[requirements],
    python_requires='>=3.7',
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
    ],
    entry_points='''
        [console_scripts]
        create-remote-repo=create_remote_repo_and_push.create_remote_repo_and_push:main
    '''
)



from abc import ABC, abstractmethod
from ..models.Repository import Repository


class BaseHandler(ABC):
    def __init__(self, next_handler=None):
        self.next_handler = next_handler

    def set_next_handler(self, next_handler):
        self.next_handler = next_handler

    def move_to_next_handler(self, request: Repository):
        if self.next_handler is not None:
            self.next_handler.process(request)
        else:
            return

    @abstractmethod
    def process(self, request: Repository):
        pass
